@@ -21,7 +21,7 @@
 #include <sys/types.h>
 #include <linux/netfilter/nfnetlink_log.h>
 
-#include <netlink-local.h>
+#include <netlink-private/netlink.h>
 #include <netlink/attr.h>
 #include <netlink/netfilter/nfnl.h>
 #include <netlink/netfilter/log.h>
